#!/usr/bin/env python3
# import logging
from dataclasses import dataclass
from datetime import date, datetime, timedelta
import logging
from typing import Any

import requests

from free import Free
from structs import Center, Coordinate, Service, validate

# logging.basicConfig(level=logging.DEBUG)

print("Running at " + str(datetime.now()))

API_URL = "https://api3.clicsante.ca/v3/"

# dateStart = "2022-01-22"
# dateStop = "2022-01-30"
# postal_code = "H3T 1K4"
# max_distance = 20 # In Km
# blacklist = [60070, 60124, 60121, 60142]
# blacklist = []
"""
140 : ITSS Standard (inclut SIDEP)
235 : Dépannage covid
236 : Dépistage covid sans symptômes
237 : 1ère dose covid
238 : 2ème dose covid
239 : 3ème dose covid
261 : 1ère dose covid enfant
262 : 2ème dose covid enfant
263 : dose supplémentaire covid
264 : distribution tests rapide covid
"""
# service_type = 140
# service_name = "Rendez-vous SIDEP"


@dataclass
class Requirements:
    date_start: date
    date_stop: date
    postal_code: str
    service_type: int
    service_name: str
    max_distance: int = 20


class Application:
    headers = {
        "authorization": "Basic cHVibGljQHRyaW1vei5jb206MTIzNDU2Nzgh",
        "x-trimoz-role": "public",
        "product": "clicsante",
    }

    def get_coordinates(self, postal_code: str) -> Coordinate:
        geoData = requests.get(
            API_URL + "geocode", params={"address": postal_code}, headers=self.headers
        )
        return Coordinate.model_validate(
            geoData.json()["results"][0]["geometry"]["location"]
        )

    def get_centers(self, requirements: Requirements) -> list[Center]:
        coordinates = self.get_coordinates(requirements.postal_code)

        centersParams = {
            "dateStart": requirements.date_start,
            "dateStop": requirements.date_stop,
            "latitude": coordinates.lat,
            "longitude": coordinates.lng,
            "maxDistance": requirements.max_distance,
            "serviceUnified": requirements.service_type,
            "postalCode": requirements.postal_code,
            "page": 0,
        }

        centers: list[Center] = []
        centersData = requests.get(
            API_URL + "availabilities", params=centersParams, headers=self.headers
        )

        while centersData.status_code == 200:
            centers += centersData.json()["places"]
            centersParams["page"] += 1
            centersData = requests.get(
                API_URL + "availabilities", params=centersParams, headers=self.headers
            )

        return validate(list[Center], centers)

    def get_services(self, establishment_id: int) -> dict[str, Any]:
        servicesData = requests.get(
            API_URL + f"establishments/{establishment_id}/services",
            headers=self.headers,
        )
        return servicesData.json()

    def find_service(self, establishment_id: int, service_name: str) -> int | None:
        services = validate(list[Service], self.get_services(establishment_id))
        for service in services:
            if service_name in service.name_fr:
                return service.id

        return None

    def get_schedule(
        self,
        center_id: int,
        establishment_id: int,
        service_id: int,
        requirements: Requirements,
    ) -> list[str]:
        scheduleParams = {
            "dateStart": requirements.date_start,
            "dateStop": requirements.date_stop,
            "service": service_id,
            "places": center_id,
            "timezone": "America/Toronto",
        }

        scheduleData = requests.get(
            API_URL + f"establishments/{establishment_id}/schedules/public",
            params=scheduleParams,
            headers=self.headers,
        )
        return scheduleData.json()["availabilities"]

    def print_schedule(self, center: Center, dates: list[str]) -> str:
        url = f"https://clients3.clicsante.ca/{center.establishment}/take-appt?unifiedService=237&portalPlace={center.id}&portalPostalCode={requirements.postal_code.replace(' ', '%20')}&lang=fr"
        result = f"Centre : {center.name_fr}\nLien : {url}\nLieu : {center.formatted_address}\nDates :\n"

        for available_date in dates:
            result += f"- {available_date}\n"

        return result

    def full_run(self, requirements: Requirements):
        centers = self.get_centers(requirements)
        for center in centers:
            service_id = self.find_service(
                center.establishment, requirements.service_name
            )
            if service_id is None:
                continue

            available_dates = self.get_schedule(
                center.id, center.establishment, service_id, requirements
            )

            if len(available_dates) > 0:
                print(self.print_schedule(center, available_dates))
                print("")


if __name__ == "__main__":
    today = date.today()

    requirements = Requirements(
        date_start=today,
        date_stop=today + timedelta(days=30),
        postal_code="H3T 1K4",
        service_name="SIDEP",
        service_type=140,
    )

    free = Free()
    application = Application()

    while True:
        center = Center(
            id=1341,
            establishment=4015,
            name_fr="SIDEP",
            formatted_address="JESAISPAS",
        )

        dates = application.get_schedule(
            center_id=center.id,
            establishment_id=center.establishment,
            service_id=1555,
            requirements=requirements,
        )

        if len(dates) > 0:
            free.send_sms(application.print_schedule(center, dates))
            break
